import 'dart:async' as Async;

import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:getx_extended/export.dart';
import 'package:getx_extended/widget/export.dart';
import 'package:rxdart/rxdart.dart' as Reactive;

extension GetExtension<T> on Rx<T> {
  ObxValue<Rx<T>> toWidget(
    Widget Function(Rx<T> rx) builder,
  ) =>
      ObxValue<Rx<T>>(builder, this);
}

extension GetReactiveExtension<T> on Rx<T> {
  Async.StreamSubscription<T> listenWithInitialValue(
          void Function(T data) onData) =>
      Future.value(value).toStream().mergeWith([subject.stream]).listen(onData);

  Async.StreamSubscription<S> combine2<A, S>({
    required Rx<A> a,
    required S Function(T t, A a) combiner,
    required Rx<S> to,
  }) =>
      Reactive.CombineLatestStream.combine2(
        subject.stream,
        a.stream,
        (t, a) => combiner(t as T, a as A),
      ).listen(to);

  Async.StreamSubscription<S> combine3<A, B, S>({
    required Rx<A> a,
    required Rx<B> b,
    required S Function(T t, A a, B b) combiner,
    required Rx<S> to,
  }) =>
      Reactive.CombineLatestStream.combine3(
        subject.stream,
        a.stream,
        b.stream,
        (t, a, b) => combiner(t as T, a as A, b as B),
      ).listen(to);
}

extension RxPageableX<T> on RxPageable<T> {
  AutoFetchList<T> toAutoFetchList({
    required Widget Function(T item) builder,
    bool reverse = false,
    Widget? header,
    Widget? separator,
    Widget? empty,
    EdgeInsetsGeometry? padding,
    VoidCallback? next,
  }) =>
      AutoFetchList<T>(
        builder: builder,
        source: source,
        reverse: reverse,
        header: header,
        separator: separator,
        empty: empty,
        padding: padding,
        next: next,
      );
}
